#!/bin/bash
###########################################
# Software Requirement:                   #
#   bash x11-utils awk wmctrl bc          #
# License:                                #
#   This script is licensed under GPL 3.0 #
# Version:                                #
#   0.4                                   #
# Author:                                 #
#   Alan CHENG <coolcd05a@gmail.com>      #
###########################################
## Custimization
# Path to essential commands
WMCTRL=`which wmctrl`
# DISTANCE_HORI is the distance to move horizontally
DISTANCE_HORI=6
# DISTANCE_VERT is the distance to move vertically
DISTANCE_VERT=6
# DISTANCE_SLASH is the distance to move when using MOVE_UR, MOVE_UL, MOVE_DR, MOVE_DL
DISTANCE_SLASH=6
# DELAY is the time delay between each pixel movement
# PAUSE is the pause between MOVE_L, MOVE_D, MOVE_R, MOVE_U...
DELAY_HORI=0.05
DELAY_VERT=0.05
DELAY_SLASH=0.05
PAUSE=1
# Screen Width and Height
RESOLUTION=`wmctrl -d|awk '/\*/{ print $4}'`
SCR_W=`echo ${RESOLUTION} |awk -F x '{ print $1 }'`
SCR_H=`echo ${RESOLUTION} |awk -F x '{ print $2 }'`
# DEP_X and DEP_Y are Allowed Pixels outside the screen
# Pixels outside top and left edges don't work,
# this may be a bug/feature of wmctrl.
DEP_X=24
DEP_Y=24

## Functions
#  POS_WINDOWS() determins the position of windows
POS_WINDOWS () {
		#L=(`${WMCTRL} -Gl`);
		#echo '$L'=$L;
		ORI_X=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left X/{ print $4; }'`
		ORI_Y=`xwininfo -id $w | awk '$0 ~ /Absolute upper-left Y/{ print $4; }'`
		#echo '$CUR_Y'=${CUR_Y};
		#echo '$CUR_X'=${CUR_X};
}

SIZE_WINDOWS () {
		CUR_W=`xwininfo -id $w | awk '$0 ~ /Width/{ print $2; }'`
		CUR_H=`xwininfo -id $w | awk '$0 ~ /Height/{ print $2; }'`
}
#  SHOW_WINDOWS() shows you the windows you can swing
SHOW_WINDOWS () {
	echo "#######################################################"
	echo " WindowID    Window Title"
	echo ""
	${WMCTRL} -l|awk '{ print $1"  "$4" "$5" "$6" "$7  }'
	echo "#######################################################"
	echo "Please select the window that you want to swing:"
	select w in `${WMCTRL} -l|awk '/^0x/{ print $1 }'`
	do W="$w" ;
		echo You have selected $w to swing ;
		POS_WINDOWS
		return;
	done
}

# MOVE_R, MOVE_D, MOVE_L, MOVE_U move the window you selected
MOVE_R () {
	for (( x=0 ; x<= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X="${ORI_X}"+"$x";
		CUR_Y="${ORI_Y}"
		if (( "${CUR_X}"+"${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )) ;then
			return;
		else
			#echo '${CUR_X}'=${CUR_X}
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_HORI};
		fi
		done;
}

MOVE_D () {
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		CUR_X="${ORI_X}"
		if (( "${CUR_Y}"+"${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_VERT};
		fi
		done;
}

MOVE_L () {
	for (( x=0 ; x <= ${DISTANCE_HORI}; x++ )) ;
		do let CUR_X="${ORI_X}"-"$x";
		CUR_Y="${ORI_Y}";
		if (( "${CUR_X}" <= 0-"${DEP_X}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_HORI};
		fi
		done;
}

MOVE_U () {
	for (( y=0 ; y<= ${DISTANCE_VERT}; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		CUR_X="${ORI_X}"
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) ;then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_VERT};
		fi
		done;
}

MOVE_UR () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		let CUR_X="${ORI_X}"+"$y";
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}"+"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_UL () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"-"$y";
		let CUR_X="${ORI_X}"-"$y";
		if (( "${CUR_Y}" <= 0-"${DEP_Y}" )) || (( "${CUR_X}" <= 0-"${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_DR () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		let CUR_X="${ORI_X}"+"$y";
		if (( "${CUR_Y}" + "${CUR_H}" >= "${SCR_H}"+"${DEP_Y}" )) || (( "${CUR_X}" + "${CUR_W}" >= "${SCR_W}" + "${DEP_X}" )); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
}

MOVE_DL () {
	for (( y=0 ; y<= `echo "${DISTANCE_SLASH}/sqrt(2)"|bc` ; y++ )) ;
		do let CUR_Y="${ORI_Y}"+"$y";
		let CUR_X="${ORI_X}"-"$y";
		if (( "$CUR_Y" + "$CUR_H" >= "${SCR_H}" + "${DEP_Y}" )) || (( "$CUR_X" <= 0 - "${DEP_X}")); then
			return;
		else
			${WMCTRL} -e 0,${CUR_X},${CUR_Y},-1,-1 -i -r $w ;
			sleep ${DELAY_SLASH};
		fi
		done;
}

case $1 in 
  hori)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	done
  ;;
  vert)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS;
	done
  ;;
  star)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS;
	done
  ;;

  triangle-cw)
  SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
;;
  triangle-ccw)
  SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
;;
  square-cw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
  ;;
  square-ccw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_L;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_D;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_R;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_U;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
  ;;
  diamond-cw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
  ;;
  diamond-ccw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS;
	MOVE_DL;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_DR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UR;
	sleep ${PAUSE}
	POS_WINDOWS;
	MOVE_UL;
	sleep ${PAUSE}
	POS_WINDOWS;
	done;
  ;;
  octagon-cw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS
	MOVE_R;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_DR;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_D;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_DL;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_L;
	POS_WINDOWS
	sleep ${PAUSE}
        MOVE_UL;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_U;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_UR;
	POS_WINDOWS
	sleep ${PAUSE}
	done
  ;;
  octagon-ccw)
	SHOW_WINDOWS
	while true
	do
	SIZE_WINDOWS
	MOVE_L;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_DL;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_D;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_DR;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_R;
	POS_WINDOWS
	sleep ${PAUSE}
        MOVE_UR;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_U;
	POS_WINDOWS
	sleep ${PAUSE}
	MOVE_UL;
	POS_WINDOWS
	sleep ${PAUSE}
	done
  ;;
  *)
	echo "Usage: /PATH_TO/swing-windows.sh {hori|vert|star|triangle-cw|triangle-ccw|square-cw|square-ccw|diamond-cw|diamond-ccw|octagon-cw|octagon-ccw}."
  ;;
esac
